// for scilab 4.1.2
//
//=================================================================
// Please edit file name !
wavfile1m='C:/Program Files/scilab-4.1.2/samplewav/sample1.wav';
wavfile1s='C:/Program Files/scilab-4.1.2/samplewav/sample2.wav';

//------------------------------------------------------------------
// wavelet relates
//------------------------------------------------------------------
function z=rem( X,Y)
  z = X - fix(X ./ Y) .*Y
endfunction

function s1 = dwt1d(s0, g, h)

//% dwt1d :  discrete wavelet transform (1-dimension)
//% s1 = dwt1d(s0, g, h)
//%
//% argument:
//%   s0: scaling coefficient level-0
//%    g: decomposition G(k)
//%    h: decomposition H(k)
//% return:
//%   s1: scaling coefficient level-1

s_length = length(s0);
s1_LPF = zeros(s_length/2, 1);
s1_HPF = zeros(s_length/2, 1);
filter_length = length(g);

for n = 0:s_length/2-1;
  for k = 0:filter_length-1;
    i = rem(2*n+k, s_length);
    s1_LPF(n+1) = s1_LPF(n+1) + g(k+1) * s0(i+1);
    s1_HPF(n+1) = s1_HPF(n+1) + h(k+1) * s0(i+1);
  end
end

for n = 0:s_length/2-1;
  s1(n+1)              = s1_LPF(n+1);
  s1(n+1 + s_length/2) = s1_HPF(n+1);
end

endfunction
//----------------------------------------------------
function s0 = idwt1d (s1, p, q)

//% idwt1d :  inverse discrete wavelet transform (1-dimension)
//% s0 = idwt1d(s1, p, q)
//%
//% argument:
//%   s1: scaling coefficient level-1
//%    p: composition P(k)
//%    q: composition Q(k)
//% return:
//%   s0: scaling coefficient level-0

s_length = length(s1);
s0_LPF = zeros(s_length, 1);
s0_HPF = zeros(s_length, 1);
filter_size = length(p);
s0 = zeros(s_length, 1);

for n = 0:s_length/2-1;
  s0_LPF(2*n+1) = s1(n+1);
  s0_HPF(2*n+1) = s1(n+1 + s_length/2);
end

for n = 0:s_length-1;
  for k = 0:filter_size-1;
    i = rem(n-k+s_length, s_length);
    s0(n+1) = s0(n+1) + p(k+1)*s0_LPF(i+1) + q(k+1)*s0_HPF(i+1) ;
  end
end

endfunction

//=================================================================
n_file =input("input 1 (sample1.wav),  2 (sample2.wav)");
if n_file == 1 then
  disp("wavfile1m");
  y0=wavread(wavfile1m);
else
 disp("wavfile1s");
 y0=wavread(wavfile1s);
end
fs=1000.0;
N0=size(y0,2);

// plot test signal out
disp0=0
wb0=xget('window');  // stack old window
xset('window',disp0);   // create new windows
clf();
plot2d(y0);
xset('window',wb0);   // push old windows


N_DIMENSION=3;

// "Daubechies"
// -- N = 1 --
if N_DIMENSION == 1 then
p0 = [0.707106781 0.707106781];
// -- N = 2nd order Wavelet
//

//---Daubechies  2nd order  Wavelet
//p0(1)=(1+sqrt(3))/8;
//p0(2)=(3+sqrt(3))/8;
//p0(3)=(3-sqrt(3))/8;
//p0(4)=(1-sqrt(3))/8;
//
//q0(1)=(1-sqrt(3))/8;
//q0(2)=-1 * (3-sqrt(3))/8;
//q0(3)=(3+sqrt(3))/8;
//q0(4)=-1 * (1+sqrt(3))/8;
//
// p0 = p0 * sqrt(2) is as same as 
// p0 = [0.482962913145 0.836516303738 0.224143868042 -0.129409522551];

elseif N_DIMENSION == 2 then
p0 = [0.482962913145 0.836516303738 0.224143868042 -0.129409522551];

elseif N_DIMENSION == 3 then
//---Daubechies  3rd order  Wavelet
p0 = [0.332670552950 0.806891509311 0.459877502118 -0.135011020010 -0.085441273882 0.035226291882];

elseif N_DIMENSION == 4 then
//---Daubechies  4th order  Wavelet
p0 = [0.230377813309 0.714846570553 0.630880767930 -0.027983769417 -0.187034811719 0.030841381836 0.032883011667 -0.010597401785];
end

sup0=length(p0);
q0 = - ( (-1).^(1:sup0) ) .* p0(sup0:-1:1);

//---------------------------------------
g = p0;
h = q0;


// 2^N no nagasa wo toridasu.
lng0=2048;
ss=y0(1:lng0);  //Origial signal
ss_stack=ss;



lng0
s1 = dwt1d(ss, g, h);
lng0/2
s2 = dwt1d(s1(1:lng0/2), g, h);
lng0/4
s3 = dwt1d(s2(1:lng0/4), g, h);
lng0/8
s4 = dwt1d(s3(1:lng0/8), g, h);
lng0/16
s5 = dwt1d(s4(1:lng0/16), g, h);
lng0/32
s6 = dwt1d(s5(1:lng0/32), g, h);
lng0/64
s7 = dwt1d(s6(1:lng0/64), g, h);
lng0/128
s8 = dwt1d(s7(1:lng0/128), g, h);
xlng0=lng0/128;



//-----------------------------
// re-construct
xlng0
is8 = idwt1d(s8, p0, q0);
iss7(1:xlng0)=is8(1:xlng0);
iss7(xlng0+1:xlng0*2)=s7(xlng0+1:xlng0*2);
xlng0=xlng0*2
is6 = idwt1d(iss7, p0, q0);
iss6(1:xlng0)=is6(1:xlng0);
iss6(xlng0+1:xlng0*2)=s6(xlng0+1:xlng0*2); 
//iss6(xlng0+1:xlng0*2)=zeros(xlng0+1:xlng0*2)';  Not This
xlng0=xlng0*2
is5 = idwt1d(iss6, p0, q0);
iss5(1:xlng0)=is5(1:xlng0);
iss5(xlng0+1:xlng0*2)=s5(xlng0+1:xlng0*2);
//iss5(xlng0+1:xlng0*2)=zeros(xlng0+1:xlng0*2)';    Not This
xlng0=xlng0*2
is4 = idwt1d(iss5, p0, q0);
iss4(1:xlng0)=is4(1:xlng0);
iss4(xlng0+1:xlng0*2)=s4(xlng0+1:xlng0*2);
iss4(xlng0+1:xlng0*2)=zeros(xlng0+1:xlng0*2)';   
xlng0=xlng0*2
is3 = idwt1d(iss4, p0, q0);
iss3(1:xlng0)=is3(1:xlng0);
iss3(xlng0+1:xlng0*2)=s3(xlng0+1:xlng0*2);
iss3(xlng0+1:xlng0*2)=zeros(xlng0+1:xlng0*2)';
xlng0=xlng0*2
is2 = idwt1d(iss3, p0, q0);
iss2(1:xlng0)=is2(1:xlng0);
iss2(xlng0+1:xlng0*2)=s2(xlng0+1:xlng0*2);
iss2(xlng0+1:xlng0*2)=zeros(xlng0+1:xlng0*2)';
xlng0=xlng0*2
is1 = idwt1d(iss2, p0, q0);
iss1(1:xlng0)=is1(1:xlng0);
iss1(xlng0+1:xlng0*2)=s1(xlng0+1:xlng0*2);
iss1(xlng0+1:xlng0*2)=zeros(xlng0+1:xlng0*2)';
is0 = idwt1d(iss1, p0, q0);

//
disp0=1
wb0=xget('window');  // stack old window
xset('window',disp0);   // create new windows
clf();

xziku0=1:1:lng0;
plot(xziku0,ss_stack,'b');
plot(xziku0,is0','r');
xset('window',wb0);   // push old windows



//----------------------------
// misc
//----------------------------
function disp_transed(ss,s1,s2,s3,s4,s5,s6,s7,s8)
disp0=2
wb0=xget('window');  // stack old window
xset('window',disp0);   // create new windows
clf();
subplot(10,1,1);
plot2d(ss);
subplot(10,1,2);
xlng0=lng0;
plot2d(s1(xlng0/2+1:xlng0));
subplot(10,1,3);
xlng0=xlng0/2;
plot2d(s2(xlng0/2+1:xlng0));
subplot(10,1,4);
xlng0=xlng0/2;
plot2d(s3(xlng0/2+1:xlng0));
subplot(10,1,5);
xlng0=xlng0/2;
plot2d(s4(xlng0/2+1:xlng0));
subplot(10,1,6);
xlng0=xlng0/2;
plot2d(s5(xlng0/2+1:xlng0));
subplot(10,1,7);
xlng0=xlng0/2;
plot2d(s6(xlng0/2+1:xlng0));
subplot(10,1,8);
xlng0=xlng0/2;
plot2d(s7(xlng0/2+1:xlng0));
subplot(10,1,9);
xlng0=xlng0/2;
plot2d(s8(xlng0/2+1:xlng0));
subplot(10,1,10);
//xlng0=xlng0/2;
plot2d(s8(1:xlng0/2));
xset('window',wb0);   // push old windows
endfunction
//-----------------------------------------